@@ -9,6 +9,8 @@ extensions = [
     "sphinx_notion",
     "sphinx_simplepdf",
     "sphinx_toolbox.collapse",
+    "sphinx_toolbox.rest_example",
     "atsphinx.audioplayer",
     "sphinx_immaterial.task_lists",
+    "sphinx.ext.mathjax",
 ]
