@@ -21,9 +21,7 @@ from sphinx.builders.text import TextBuilder
 from sphinx.util import docutils as sphinx_docutils
 from sphinx.util import logging as sphinx_logging
 from sphinx.util.typing import ExtensionMetadata
-from sphinx_immaterial.task_lists import (  # pyright: ignore[reportMissingTypeStubs]
-    checkbox_label,
-)
+from sphinx_immaterial.task_lists import checkbox_label
 from sphinx_simplepdf.directives.pdfinclude import (  # pyright: ignore[reportMissingTypeStubs]
     PdfIncludeDirective,
 )
@@ -39,6 +37,7 @@ from ultimate_notion.blocks import Block, ParentBlock
 from ultimate_notion.blocks import BulletedItem as UnoBulletedItem
 from ultimate_notion.blocks import Callout as UnoCallout
 from ultimate_notion.blocks import Code as UnoCode
+from ultimate_notion.blocks import Equation as UnoEquation
 from ultimate_notion.blocks import Heading as UnoHeading
 from ultimate_notion.blocks import (
     Heading1 as UnoHeading1,
@@ -68,20 +67,17 @@ from ultimate_notion.blocks import (
 from ultimate_notion.blocks import Video as UnoVideo
 from ultimate_notion.file import ExternalFile
 from ultimate_notion.obj_api.enums import BGColor, CodeLang, Color
-from ultimate_notion.rich_text import Text, text
+from ultimate_notion.rich_text import Text, math, text
 
 _LOGGER = sphinx_logging.getLogger(name=__name__)
 
 
 @beartype
-def _color_from_node(*, node: nodes.inline) -> Color | None:
-    """Extract Notion color from CSS classes.
-
-    ``sphinxcontrib-text-styles`` creates classes like 'text-red', 'text-
-    blue', etc.
+def _get_text_color_mapping() -> dict[str, Color]:
     """
-    classes = node.attributes.get("classes", [])
-    color_mapping: dict[str, Color] = {
+    Get the mapping from CSS classes to Notion colors.
+    """
+    return {
         "text-red": Color.RED,
         "text-blue": Color.BLUE,
         "text-green": Color.GREEN,
@@ -94,17 +90,65 @@ def _color_from_node(*, node: nodes.inline) -> Color | None:
         "text-grey": Color.GRAY,
     }
 
+
+@beartype
+def _get_background_color_classes() -> set[str]:
+    """
+    Get the set of supported background color classes.
+    """
+    return {
+        "bg-red",
+        "bg-blue",
+        "bg-green",
+        "bg-yellow",
+        "bg-orange",
+        "bg-purple",
+        "bg-pink",
+        "bg-brown",
+        "bg-gray",
+        "bg-grey",
+    }
+
+
+@beartype
+def _color_from_css_classes(*, classes: list[str]) -> Color | None:
+    """Extract Notion color from CSS classes.
+
+    Classes created by ``sphinxcontrib-text-styles``.
+    """
+    color_mapping = _get_text_color_mapping()
+
     for css_class in classes:
         if css_class in color_mapping:
             return color_mapping[css_class]
 
-        # This warning being here assumes that only color classes,
-        # and only classes from ``sphinxcontrib-text-styles``, are used.
-        _LOGGER.warning(
-            "Unsupported text style classes: %s. "
-            "Text will be rendered without styling.",
-            css_class,
-        )
+    return None
+
+
+@beartype
+def _background_color_from_css_classes(
+    *, classes: list[str]
+) -> BGColor | None:
+    """Extract Notion background color from CSS classes.
+
+    Classes created by ``sphinxcontrib-text-styles``.
+    """
+    bg_color_mapping: dict[str, BGColor] = {
+        "bg-red": BGColor.RED,
+        "bg-blue": BGColor.BLUE,
+        "bg-green": BGColor.GREEN,
+        "bg-yellow": BGColor.YELLOW,
+        "bg-orange": BGColor.ORANGE,
+        "bg-purple": BGColor.PURPLE,
+        "bg-pink": BGColor.PINK,
+        "bg-brown": BGColor.BROWN,
+        "bg-gray": BGColor.GRAY,
+        "bg-grey": BGColor.GRAY,
+    }
+
+    for css_class in classes:
+        if css_class in bg_color_mapping:
+            return bg_color_mapping[css_class]
 
     return None
 
@@ -160,6 +204,200 @@ class _TableStructure:
     num_stub_columns: int
 
 
+@singledispatch
+@beartype
+def _process_rich_text_node(child: nodes.Node) -> Text:
+    """Create Notion rich text from a single ``docutils`` node.
+
+    This is the base function for ``singledispatch``. Specific node types
+    are handled by registered functions.
+    """
+    unsupported_child_type_msg = (
+        f"Unsupported node type within text: {type(child).__name__} on line "
+        f"{child.parent.line} in {child.parent.source}."
+    )
+    # We use ``TRY004`` here because we want to raise a
+    # ``ValueError`` if the child type is unsupported, not a
+    # ``TypeError`` as the user has not directly provided any type.
+    raise ValueError(unsupported_child_type_msg)
+
+
+@beartype
+@_process_rich_text_node.register
+def _(child: nodes.reference) -> Text:
+    """
+    Process reference nodes by creating linked text.
+    """
+    link_url = child.attributes["refuri"]
+    link_text = child.attributes.get("name", link_url)
+
+    return text(
+        text=link_text,
+        href=link_url,
+        bold=False,
+        italic=False,
+        code=False,
+    )
+
+
+@beartype
+@_process_rich_text_node.register
+def _(child: nodes.target) -> Text:
+    """
+    Process target nodes by returning empty text (targets are skipped).
+    """
+    del child  # Target nodes are skipped
+    return Text.from_plain_text(text="")
+
+
+@beartype
+@_process_rich_text_node.register
+def _(child: nodes.title_reference) -> Text:
+    """Process title reference nodes by creating italic text.
+
+    We match the behavior of the HTML builder here.
+    If you render ``A `B``` in HTML, it will render as ``A <i>B</i>``.
+    """
+    return text(text=child.astext(), italic=True)
+
+
+@beartype
+@_process_rich_text_node.register
+def _(child: nodes.Text) -> Text:
+    """
+    Process Text nodes by creating plain text.
+    """
+    return text(text=child.astext())
+
+
+@beartype
+@_process_rich_text_node.register
+def _(child: nodes.inline) -> Text:
+    """
+    Process inline nodes by creating styled text.
+    """
+    return _create_styled_text_from_node(child=child)
+
+
+@beartype
+@_process_rich_text_node.register
+def _(child: nodes.strong) -> Text:
+    """
+    Process strong nodes by creating bold text.
+    """
+    return _create_styled_text_from_node(child=child)
+
+
+@beartype
+@_process_rich_text_node.register
+def _(child: nodes.emphasis) -> Text:
+    """
+    Process emphasis nodes by creating italic text.
+    """
+    return _create_styled_text_from_node(child=child)
+
+
+@beartype
+@_process_rich_text_node.register
+def _(child: nodes.literal) -> Text:
+    """
+    Process literal nodes by creating code text.
+    """
+    return _create_styled_text_from_node(child=child)
+
+
+@beartype
+@_process_rich_text_node.register
+def _(child: strike_node) -> Text:
+    """
+    Process strike nodes by creating strikethrough text.
+    """
+    return _create_styled_text_from_node(child=child)
+
+
+@beartype
+@_process_rich_text_node.register
+def _(child: nodes.paragraph) -> Text:
+    """
+    Process paragraph nodes by creating styled text.
+    """
+    return _create_styled_text_from_node(child=child)
+
+
+@beartype
+@_process_rich_text_node.register
+def _(child: nodes.math) -> Text:
+    """
+    Process math nodes by creating math rich text.
+    """
+    return math(expression=child.astext())
+
+
+@beartype
+def _create_styled_text_from_node(*, child: nodes.Element) -> Text:
+    """Create styled text from a node with CSS class support.
+
+    This helper function handles the complex styling logic that was
+    previously inline in the main function.
+    """
+    classes = child.attributes.get("classes", [])
+    bg_color = _background_color_from_css_classes(classes=classes)
+    text_color = _color_from_css_classes(classes=classes)
+
+    color_mapping = _get_text_color_mapping()
+    bg_color_classes = _get_background_color_classes()
+
+    is_bold = isinstance(child, nodes.strong) or "text-bold" in classes
+    is_italic = isinstance(child, nodes.emphasis) or "text-italic" in classes
+    is_code = (
+        isinstance(child, nodes.literal)
+        or "text-mono" in classes
+        or "kbd" in classes
+    )
+    is_strikethrough = (
+        isinstance(child, strike_node) or "text-strike" in classes
+    )
+    is_underline = "text-underline" in classes
+
+    supported_style_classes = {
+        "text-bold",
+        "text-italic",
+        "text-mono",
+        "text-strike",
+        "text-underline",
+        "kbd",
+        *color_mapping.keys(),
+        *bg_color_classes,
+    }
+    unsupported_styles = [
+        css_class
+        for css_class in classes
+        if css_class not in supported_style_classes
+    ]
+
+    if unsupported_styles:
+        unsupported_style_msg = (
+            "Unsupported text style classes: "
+            f"{', '.join(unsupported_styles)}. "
+            f"Text on line {child.parent.line} in {child.parent.source} will "
+            "be rendered without styling."
+        )
+        _LOGGER.warning(unsupported_style_msg)
+
+    color: BGColor | Color | None = bg_color or text_color
+    return text(
+        text=child.astext(),
+        bold=is_bold,
+        italic=is_italic,
+        code=is_code,
+        strikethrough=is_strikethrough,
+        underline=is_underline,
+        # Ignore the type check here because Ultimate Notion has
+        # a bad type hint: https://github.com/ultimate-notion/ultimate-notion/issues/140
+        color=color,  # type: ignore[arg-type]  # pyright: ignore[reportArgumentType]
+    )
+
+
 @beartype
 def _create_rich_text_from_children(*, node: nodes.Element) -> Text:
     """Create Notion rich text from ``docutils`` node children.
@@ -172,36 +410,7 @@ def _create_rich_text_from_children(*, node: nodes.Element) -> Text:
     rich_text = Text.from_plain_text(text="")
 
     for child in node.children:
-        if isinstance(child, nodes.reference):
-            link_url = child.attributes["refuri"]
-            link_text = child.attributes.get("name", link_url)
-
-            new_text = text(
-                text=link_text,
-                href=link_url,
-                bold=False,
-                italic=False,
-                code=False,
-            )
-        elif isinstance(child, nodes.target):
-            continue
-        elif isinstance(child, nodes.inline):
-            new_text = text(
-                text=child.astext(),
-                bold=isinstance(child, nodes.strong),
-                italic=isinstance(child, nodes.emphasis),
-                code=isinstance(child, nodes.literal),
-                strikethrough=isinstance(child, strike_node),
-                color=_color_from_node(node=child),
-            )
-        else:
-            new_text = text(
-                text=child.astext(),
-                bold=isinstance(child, nodes.strong),
-                italic=isinstance(child, nodes.emphasis),
-                code=isinstance(child, nodes.literal),
-                strikethrough=isinstance(child, strike_node),
-            )
+        new_text = _process_rich_text_node(child)
         rich_text += new_text
 
     return rich_text
@@ -369,6 +578,9 @@ def _map_pygments_to_notion_language(*, pygments_lang: str) -> CodeLang:
         "reason": CodeLang.REASON,
         "ruby": CodeLang.RUBY,
         "rb": CodeLang.RUBY,
+        # This is not a perfect match, but at least rest-example will
+        # be rendered.
+        "rest": CodeLang.PLAIN_TEXT,
         "rust": CodeLang.RUST,
         "rs": CodeLang.RUST,
         "sass": CodeLang.SASS,
@@ -408,12 +620,17 @@ def _process_node_to_blocks(
     node: nodes.Element,
     *,
     section_level: int,
-) -> list[Block]:  # pragma: no cover
+) -> list[Block]:
     """
     Required function for ``singledispatch``.
     """
     del section_level
-    raise NotImplementedError(node)
+    unsupported_node_type_msg = (
+        f"Unsupported node type: {node.tagname} on line "
+        f"{node.line or node.parent.line} in "
+        f"{node.source or node.parent.source}."
+    )
+    raise NotImplementedError(unsupported_node_type_msg)
 
 
 @beartype
@@ -459,9 +676,15 @@ def _(
         _LOGGER.warning(msg=table_multiple_header_rows_msg)
 
     if table_structure.num_stub_columns > 1:
+        first_body_row = table_structure.body_rows[0]
+        first_body_row_entry = first_body_row.children[0]
+        first_body_row_paragraph = first_body_row_entry.children[0]
         table_more_than_one_stub_column_msg = (
             f"Tables with more than 1 stub column are not supported. "
-            f"Found {table_structure.num_stub_columns} stub columns."
+            f"Found {table_structure.num_stub_columns} stub columns "
+            f"on table with first body row on line "
+            f"{first_body_row_paragraph.line} in "
+            f"{first_body_row_paragraph.source}."
         )
         _LOGGER.warning(msg=table_more_than_one_stub_column_msg)
 
@@ -491,10 +714,25 @@ def _(
     *,
     section_level: int,
 ) -> list[Block]:
+    """Process paragraph nodes by creating Notion Paragraph blocks.
+
+    Special case: if the paragraph contains only a container a
+    ``rest-example`` class, process the container directly instead of
+    trying to process it as rich text.
     """
-    Process paragraph nodes by creating Notion Paragraph blocks.
-    """
-    del section_level
+    if (
+        len(node.children) == 1
+        and isinstance(
+            node.children[0],
+            nodes.container,
+        )
+        and node.children[0].attributes.get("classes", []) == ["rest-example"]
+    ):
+        return _process_node_to_blocks(
+            node.children[0],
+            section_level=section_level,
+        )
+
     rich_text = _create_rich_text_from_children(node=node)
     return [UnoParagraph(text=rich_text)]
 
@@ -1073,7 +1311,7 @@ def _(
     section_level: int,
 ) -> list[Block]:
     """
-    Process container nodes, especially for ``literalinclude`` with captions.
+    Process container nodes.
     """
     num_children_for_captioned_literalinclude = 2
     if (
@@ -1100,6 +1338,13 @@ def _(
             )
         ]
 
+    classes = node.attributes.get("classes", [])
+    if classes == ["rest-example"]:
+        return _process_rest_example_container(
+            node=node,
+            section_level=section_level,
+        )
+
     blocks: list[Block] = []
     for child in node.children:
         child_blocks = _process_node_to_blocks(
@@ -1109,6 +1354,38 @@ def _(
     return blocks
 
 
+@beartype
+def _process_rest_example_container(
+    *,
+    node: nodes.container,
+    section_level: int,
+) -> list[Block]:
+    """
+    Process a ``rest-example`` container by creating nested callout blocks.
+    """
+    rst_source_node = node.children[0]
+    assert isinstance(rst_source_node, nodes.literal_block)
+    output_nodes = node.children[1:]
+    code_blocks = _process_node_to_blocks(rst_source_node, section_level=1)
+
+    output_blocks: list[Block] = []
+    for output_node in output_nodes:
+        output_blocks.extend(
+            _process_node_to_blocks(output_node, section_level=section_level)
+        )
+
+    code_callout = UnoCallout(text=text(text="Code"))
+    code_callout.append(blocks=code_blocks)
+
+    output_callout = UnoCallout(text=text(text="Output"))
+    output_callout.append(blocks=output_blocks)
+
+    main_callout = UnoCallout(text=text(text="Example"))
+    main_callout.append(blocks=[code_callout, output_callout])
+
+    return [main_callout]
+
+
 @beartype
 @_process_node_to_blocks.register
 def _(
@@ -1125,6 +1402,36 @@ def _(
     return []
 
 
+@beartype
+@_process_node_to_blocks.register
+def _(
+    node: nodes.math_block,
+    *,
+    section_level: int,
+) -> list[Block]:
+    """
+    Process math block nodes by creating Notion Equation blocks.
+    """
+    del section_level
+    latex_content = node.astext()
+    return [UnoEquation(latex=latex_content)]
+
+
+@beartype
+@_process_node_to_blocks.register
+def _(
+    node: nodes.target,
+    *,
+    section_level: int,
+) -> list[Block]:
+    """
+    Process target nodes by ignoring them completely.
+    """
+    del node
+    del section_level
+    return []
+
+
 @beartype
 @_process_node_to_blocks.register
 def _(
