@@ -21,6 +21,7 @@ from ultimate_notion.blocks import Block, ParentBlock
 from ultimate_notion.blocks import BulletedItem as UnoBulletedItem
 from ultimate_notion.blocks import Callout as UnoCallout
 from ultimate_notion.blocks import Code as UnoCode
+from ultimate_notion.blocks import Equation as UnoEquation
 from ultimate_notion.blocks import (
     Heading1 as UnoHeading1,
 )
@@ -49,7 +50,7 @@ from ultimate_notion.blocks import (
 from ultimate_notion.blocks import Video as UnoVideo
 from ultimate_notion.file import ExternalFile
 from ultimate_notion.obj_api.enums import BGColor, CodeLang, Color
-from ultimate_notion.rich_text import text
+from ultimate_notion.rich_text import Text, math, text
 
 
 @beartype
@@ -1998,7 +1999,8 @@ def test_list_table_stub_columns_two(
 
     expected_warning = (
         "Tables with more than 1 stub column are not supported. "
-        "Found 2 stub columns."
+        "Found 2 stub columns on table with first body row on line 8 in "
+        f"{tmp_path / 'src' / 'index.rst'}."
     )
 
     table = UnoTable(n_rows=3, n_cols=3, header_row=True, header_col=True)
@@ -2255,35 +2257,49 @@ and :text-green:`green text`.
 
 
 @pytest.mark.parametrize(
-    argnames=("color_name", "expected_color"),
+    argnames=("role", "expected_color"),
     argvalues=[
-        ("red", Color.RED),
-        ("blue", Color.BLUE),
-        ("green", Color.GREEN),
-        ("yellow", Color.YELLOW),
-        ("orange", Color.ORANGE),
-        ("purple", Color.PURPLE),
-        ("pink", Color.PINK),
-        ("brown", Color.BROWN),
-        ("gray", Color.GRAY),
+        ("text-red", Color.RED),
+        ("text-blue", Color.BLUE),
+        ("text-green", Color.GREEN),
+        ("text-yellow", Color.YELLOW),
+        ("text-orange", Color.ORANGE),
+        ("text-purple", Color.PURPLE),
+        ("text-pink", Color.PINK),
+        ("text-brown", Color.BROWN),
+        ("text-gray", Color.GRAY),
+        ("bg-red", BGColor.RED),
+        ("bg-blue", BGColor.BLUE),
+        ("bg-green", BGColor.GREEN),
+        ("bg-yellow", BGColor.YELLOW),
+        ("bg-orange", BGColor.ORANGE),
+        ("bg-purple", BGColor.PURPLE),
+        ("bg-pink", BGColor.PINK),
+        ("bg-brown", BGColor.BROWN),
+        ("bg-gray", BGColor.GRAY),
     ],
 )
 def test_individual_colors(
     *,
     make_app: Callable[..., SphinxTestApp],
     tmp_path: Path,
-    color_name: str,
-    expected_color: Color,
+    role: str,
+    expected_color: Color | BGColor,
 ) -> None:
     """
     Each supported color is converted correctly.
     """
     rst_content = f"""
-        This is :text-{color_name}:`{color_name} text`.
+        This is :{role}:`{role} text`.
     """
 
     normal_text = text(text="This is ")
-    colored_text = text(text=f"{color_name} text", color=expected_color)
+    colored_text = text(
+        text=f"{role} text",
+        # We ignore the type check here because Ultimate Notion has
+        # a bad type hint: https://github.com/ultimate-notion/ultimate-notion/issues/140
+        color=expected_color,  # type: ignore[arg-type]  # pyright: ignore[reportArgumentType]
+    )
     normal_text2 = text(text=".")
 
     combined_text = normal_text + colored_text + normal_text2
@@ -2301,28 +2317,29 @@ def test_individual_colors(
     )
 
 
-def test_text_styles_non_color(
+def test_text_styles_unsupported_color(
     *,
     make_app: Callable[..., SphinxTestApp],
     tmp_path: Path,
 ) -> None:
     """
-    Non-color text styles (like underline) emit a warning.
+    Unsupported colors from ``sphinxcontrib-text-styles`` emit warnings.
     """
     rst_content = """
-        This is :text-underline:`underlined text`.
+        This is :text-cyan:`cyan text`.
     """
 
     expected_warning = (
-        "Unsupported text style classes: text-underline. "
-        "Text will be rendered without styling."
+        "Unsupported text style classes: text-cyan. "
+        f"Text on line 1 in {tmp_path / 'src' / 'index.rst'} will be rendered "
+        "without styling."
     )
 
     normal_text = text(text="This is ")
-    underline_text = text(text="underlined text")
+    cyan_text = text(text="cyan text")
     normal_text2 = text(text=".")
 
-    combined_text = normal_text + underline_text + normal_text2
+    combined_text = normal_text + cyan_text + normal_text2
 
     expected_paragraph = UnoParagraph(text=combined_text)
 
@@ -2426,6 +2443,49 @@ def test_text_styles_and_strike(
     )
 
 
+@pytest.mark.parametrize(
+    argnames=("role", "expected_text"),
+    argvalues=[
+        ("text-bold", text(text="text-bold text", bold=True)),
+        ("text-italic", text(text="text-italic text", italic=True)),
+        ("text-mono", text(text="text-mono text", code=True)),
+        ("text-strike", text(text="text-strike text", strikethrough=True)),
+        ("text-underline", text(text="text-underline text", underline=True)),
+    ],
+)
+def test_additional_text_styles(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+    role: str,
+    expected_text: Text,
+) -> None:
+    """
+    Additional text styles from the ``sphinxcontrib_text_styles`` extension are
+    supported.
+    """
+    rst_content = f"""
+        This is :{role}:`{role} text`.
+    """
+
+    normal_text1 = text(text="This is ")
+    normal_text2 = text(text=".")
+
+    combined_text = normal_text1 + expected_text + normal_text2
+
+    expected_paragraph = UnoParagraph(text=combined_text)
+
+    expected_objects: list[Block] = [expected_paragraph]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        extensions=("sphinx_notion", "sphinxcontrib_text_styles"),
+    )
+
+
 def test_flat_task_list(
     *,
     make_app: Callable[..., SphinxTestApp],
@@ -2581,8 +2641,7 @@ def test_nested_task_list(
                   * [x] Task B2
                   * [ ] Task B3
 
-              A rogue paragraph with a reference to
-              the `parent task_list <task_list_example>`.
+              A rogue paragraph.
 
               - A list item without a checkbox.
               - [ ] Another bullet point.
@@ -2603,11 +2662,7 @@ def test_nested_task_list(
     # The rogue paragraph is nested within Task B
     # Note: The actual output has the text split across multiple rich text
     # segments
-    rogue_paragraph = UnoParagraph(
-        text=text(text="A rogue paragraph with a reference to\nthe ")
-        + text(text="parent task_list <task_list_example>")
-        + text(text=".")
-    )
+    rogue_paragraph = UnoParagraph(text=text(text="A rogue paragraph."))
     task_b.append(blocks=[rogue_paragraph])
 
     # Regular bullet list items should be nested within Task B as bullet items
@@ -2668,3 +2723,292 @@ def test_task_list_quote(
         tmp_path=tmp_path,
         extensions=("sphinx_notion", "sphinx_immaterial.task_lists"),
     )
+
+
+def test_inline_single_backticks(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    Reproduces a bug where we got confused by mismatching blocks.
+    """
+    rst_content = """
+        A `B`
+    """
+    expected_objects: list[Block] = [
+        UnoParagraph(text=text(text="A ") + text(text="B", italic=True)),
+    ]
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+    )
+
+
+def test_kbd_role(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """The ``:kbd:`` role creates keyboard input formatting.
+
+    The ``:kbd:`` role splits keyboard shortcuts into separate segments,
+    where each key is formatted as code but the + separator is not.
+    """
+    rst_content = """
+        Press :kbd:`Ctrl+C` to copy and :kbd:`Ctrl+V` to paste.
+    """
+
+    normal_text1 = text(text="Press ")
+    kbd_text1 = text(text="Ctrl", code=True)
+    plus_text1 = text(text="+", code=False)
+    kbd_text2 = text(text="C", code=True)
+    normal_text2 = text(text=" to copy and ")
+    kbd_text3 = text(text="Ctrl", code=True)
+    plus_text2 = text(text="+", code=False)
+    kbd_text4 = text(text="V", code=True)
+    normal_text3 = text(text=" to paste.")
+
+    combined_text = (
+        normal_text1
+        + kbd_text1
+        + plus_text1
+        + kbd_text2
+        + normal_text2
+        + kbd_text3
+        + plus_text2
+        + kbd_text4
+        + normal_text3
+    )
+
+    expected_paragraph = UnoParagraph(text=combined_text)
+
+    expected_objects: list[Block] = [expected_paragraph]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+    )
+
+
+def test_unsupported_node_types_in_rich_text(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    Unsupported node types in rich text processing raise ValueError.
+    """
+    rst_content = """
+        This is a test with :footnote:`footnote node`.
+    """
+
+    conf_py_content = """
+from docutils import nodes
+
+def setup(app):
+    def footnote_role(
+        name, rawtext, text, lineno, inliner, options={}, content=[]
+    ):  # noqa: PLR0913
+        node = nodes.footnote_reference(rawtext, text)
+        return [node], []
+
+    app.add_role('footnote', footnote_role)
+    """
+    expected_message = (
+        r"^Unsupported node type within text: footnote_reference on line 1 in "
+        rf"{re.escape(pattern=str(object=tmp_path / 'src' / 'index.rst'))}\.$"
+    )
+    with pytest.raises(expected_exception=ValueError, match=expected_message):
+        _assert_rst_converts_to_notion_objects(
+            rst_content=rst_content,
+            expected_objects=[],
+            make_app=make_app,
+            tmp_path=tmp_path,
+            conf_py_content=conf_py_content,
+        )
+
+
+def test_unsupported_node_types_in_process_node_to_blocks(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    Unsupported node types in _process_node_to_blocks raise
+    NotImplementedError.
+    """
+    rst_content = """
+        .. unsupported-node::
+    """
+
+    conf_py_content = """
+from docutils import nodes
+from docutils.parsers.rst import Directive
+
+class UnsupportedNode(Directive):
+    def run(self):
+        # Create a custom node type that's not registered
+        node = nodes.Element()
+        node.tagname = 'unsupported_node'
+        return [node]
+
+def setup(app):
+    app.add_directive('unsupported-node', UnsupportedNode)
+    """
+
+    expected_message = (
+        r"^Unsupported node type: unsupported_node on line "
+        rf"1 in "
+        rf"{re.escape(pattern=str(object=tmp_path / 'src' / 'index.rst'))}.$"
+    )
+    with pytest.raises(
+        expected_exception=NotImplementedError,
+        match=expected_message,
+    ):
+        _assert_rst_converts_to_notion_objects(
+            rst_content=rst_content,
+            expected_objects=[],
+            make_app=make_app,
+            tmp_path=tmp_path,
+            conf_py_content=conf_py_content,
+        )
+
+
+def test_inline_equation(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    Inline equations become Notion math rich text.
+    """
+    rst_content = """
+        This is an inline equation :math:`E = mc^2` in a paragraph.
+    """
+
+    normal_text1 = text(text="This is an inline equation ")
+    equation_text = math(expression="E = mc^2")
+    normal_text2 = text(text=" in a paragraph.")
+
+    combined_text = normal_text1 + equation_text + normal_text2
+
+    expected_paragraph = UnoParagraph(text=combined_text)
+
+    expected_objects: list[Block] = [expected_paragraph]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        extensions=("sphinx_notion", "sphinx.ext.mathjax"),
+    )
+
+
+def test_block_equation(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    Block equations become Notion Equation blocks.
+    """
+    rst_content = """
+        .. math::
+
+           E = mc^2
+    """
+
+    expected_objects: list[Block] = [
+        UnoEquation(latex="E = mc^2"),
+    ]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        extensions=("sphinx_notion", "sphinx.ext.mathjax"),
+    )
+
+
+def test_rest_example_block(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    Rest example blocks become Notion callout blocks with nested code and
+    description.
+    """
+    rst_content = """
+        .. rest-example::
+
+           .. code-block:: python
+
+              def hello_world():
+                  print("Hello, World!")
+
+           Rendered output shows what the code does.
+    """
+
+    code_callout = UnoCallout(
+        text=text(text="Code"),
+    )
+    code_callout.append(
+        blocks=[
+            UnoCode(
+                text=text(
+                    text=textwrap.dedent(
+                        text="""\
+                        .. code-block:: python
+
+                           def hello_world():
+                               print("Hello, World!")
+
+                        Rendered output shows what the code does."""
+                    )
+                ),
+                language="plain text",
+            ),
+        ]
+    )
+
+    output_callout = UnoCallout(
+        text=text(text="Output"),
+    )
+    output_callout.append(
+        blocks=[
+            UnoCode(
+                text=text(
+                    text=textwrap.dedent(
+                        text="""\
+                        def hello_world():
+                            print("Hello, World!")""",
+                    )
+                ),
+                language="python",
+            ),
+            UnoParagraph(
+                text=text(text="Rendered output shows what the code does.")
+            ),
+        ]
+    )
+
+    main_callout = UnoCallout(text=text(text="Example"))
+    main_callout.append(blocks=[code_callout, output_callout])
+
+    expected_objects: list[Block] = [main_callout]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        extensions=("sphinx_notion", "sphinx_toolbox.rest_example"),
+    )
